from .base import BaseException as error


class UnauthorizedException(error):
    """_summary_: UnauthorizedException
    _description_: The unauthorized exception for thepeer sdk
    _usage_:
        raise UnauthorizedException(errors)
    _example_:
        raise UnauthorizedException({message: "who are you?"})
    """

    def __init__(self, errors):
        super().__init__(errors)



from .base import BaseException as error


class NotFoundException(error):
    """_summary_: NotFoundException
    _description_: The not found exception for thepeer sdk
    _usage_:
        raise NotFoundException(errors)
    _example_:
        raise NotFoundException({message: "you are definitely lost"})
    """

    def __init__(self, errors):
        super().__init__(errors)



from .base import BaseException as error


class ServiceUnavailableException(error):
    """_summary_: ServiceUnavailableException
    _description_: The service unavailable exception for thepeer sdk
    _usage_:
        raise ServiceUnavailableException(errors)
    _example_:
        raise ServiceUnavailableException({message: "apparently our servers are taking a little vacation of reality"})
    """

    def __init__(self, errors):
        super().__init__(errors)



# the base error for thepeer sdk inheriting from the Exception class
class BaseException(Exception):
    """_summary_: BaseException
    _description_: The base exception for thepeer sdk
    _usage_:
        raise BaseException(errors)
    _example_:
        raise BaseException({message: "an error occurred"})
    """

    def __init__(self, errors={}):
        self.errors = errors

    def __str__(self):
        return self.errors



import hmac
import hashlib
import json  # type: ignore
import os
import sys

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


import httpx  # noqa: E402
from utils.constants import BASE_URL  # noqa: E402
from utils.exceptions.handleErrors import SwitchErrorStates  # noqa: E402


class ThepeerInit:
    def __init__(self, secret):
        # pass a default value for the url
        self.url = BASE_URL
        self.secret = secret
        # set default headers to be used in all requests
        self.headers = {"x-api-key": self.secret, "content-Type": "application/json"}

    def validate_signature(self, data, signature):
        """helper method to validate the signature of the data received
        from thepeer's servers, it's usually used for validating webhook events
        coming from thepeer's servers
        Args:
            data (dict|Any): the payload to which the signature is applied
            signature (string): the signature to be validated

        Returns:
            _type_: boolean if the signature is valid or not
        """
        SHA1 = hashlib.sha1
        return signature == hmac.new(self.secret.encode(), data.encode(), SHA1).hexdigest()

    def index_user(self, name, identifier, email):

        """this method helps identify the user on thepeer's servers in order to facilitate
        more usage of the SDK
        it is usually the first method called by the user

        Args:
            name (string): the name of the registered user
            identifier (string): the universal identifier of the registered user
            email (string): the email of the registered user

        Returns:
            dict: the json response from the server containing the user's id and
            other related information
        """
        try:

            data = {"name": name, "identifier": identifier, "email": email}
            # convert the data to json
            data = json.dumps(data)
            response = httpx.post(f"{self.url}/users", data=data, headers=dict(self.headers))
            return response.json()

        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def view_user(self, reference):
        """this method helps view the user's information on thepeer's servers
        it is usually called after the user has indexed himself

        Args:
            reference (string): the reference of the indexed user

        Returns:
            dict: the json response from the server containing the user's id and
            other related information
        """
        try:
            response = httpx.get(f"{self.url}/users/{reference}", headers=dict(self.headers))
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def all_users(self, page=1, per_page=15):
        """this method gets all the indexed users of a business

        Args:
            page (int): a specific page of pagination instances
            per_page (int): number of users to display in a single paginated instance page

        Returns:
            dict: a dict containing paginated lists of dicts of indexed users
        """

        try:
            response = httpx.get(
                f"{self.url}/users?page={page}&perPage={per_page}", headers=dict(self.headers)
            )
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def update_user(self, reference, **data):
        """this method helps update the user's information on thepeer's servers
        it is usually called after the user has indexed himself

        Args:
            reference (string): the reference of the indexed user
            data (dict): the data to be updated which is a dictionary
            of at least one of the fields (name, identifier, identifier_type,email)

        Returns:
            dict: the json response from the server containing the user's id and
            other related information
        """
        try:
            data = json.dumps(data)
            response = httpx.put(
                f"{self.url}/users/{reference}", data=data, headers=dict(self.headers)
            )
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def delete_user(self, reference):
        """this method helps delete the indexed user info on thepeer's servers
        a user can always reindex himself

        Args:
            reference (string): the reference of the indexed user
        """
        try:
            response = httpx.delete(f"{self.url}/users/{reference}", headers=dict(self.headers))
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def get_user_links(self, reference):
        """this method helps to get the user's payment links on thepeer's servers


        Args:
            reference (string): the reference of the indexed user

        Returns:
            dict: the json response from the server containing the linked accounts
            related to the user's id and other related information
        """
        try:
            response = httpx.get(f"{self.url}/users/{reference}/links", headers=dict(self.headers))
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def get_single_link(self, link_id):
        """this method returns a user's linked account details on thepeer's servers


        Args:
            link_id (string): the link ID

        Returns:
            dict: the json response from the server containing the payment link id and
            other related information
        """
        try:
            response = httpx.get(f"{self.url}/link/{link_id}", headers=dict(self.headers))
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def charge_link(self, link_id, amount, remark, currency="NGN"):
        """allows a business to charge a user via their linked account

        Args:
            link_id (string):the link ID
            data (dict): the request payload which contains the amount and remark and optionally,
            a currency (which could be NGN or USD default being NGN)

        Returns:
            dict: a transaction object generated from a webhook. more info about a
            transaction object here https://docs.thepeer.co/transaction/transaction-object
        """
        try:
            data = json.dumps({"amount": amount, "remark": remark, "currency": currency})
            response = httpx.post(
                f"{self.url}/link/{link_id}/charge", data=data, headers=dict(self.headers)
            )
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def authorize_direct_charge(self, auth_charge_reference, event):
        """allows a business to authorize a direct charge request made by a user

        Args:
            auth_charge_reference (string): reference generated from the direct charge webhook event
            event (string): a string which contains the kind of webhook event

        Returns:
            dict: containing a key value pair of the event(key) and the type of event
        """
        try:
            data = json.dumps({"event": event})
            response = httpx.post(
                f"{self.url}/authorization/{auth_charge_reference}",
                data=data,
                headers=dict(self.headers),
            )
            return response.json()

        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def get_transaction_detail(self, transaction_id):
        """Get the details about a transaction
        Args:
            transaction_id (string): the unique identifier of the transaction

        Returns:
            dict:containing the transaction object. more about a transaction object here
            https://docs.thepeer.co/transaction/transaction-object#anatomy-of-a-transaction-object
        """
        try:
            response = httpx.get(
                f"{self.url}/transactions/{transaction_id}", headers=dict(self.headers)
            )
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()

    def refund_transaction(self, transaction_id, reason):

        """This method allows a business to refund a transaction back to the user
        who made the transaction for obvious reasons.

        Args:
            transaction_id (string): the unique identifier of the transaction
            reason (string): a string explaining the reason for the refund

        Returns:
             dict:
             containing the transaction object. more about a transaction object can be found here
             https://docs.thepeer.co/transaction/transaction-object#anatomy-of-a-transaction-object
        """
        try:
            data = json.dumps({"reason": reason})
            response = httpx.post(
                f"{self.url}/transactions/{transaction_id}/refund",
                data=data,
                headers=dict(self.headers),
            )
            return response.json()
        except Exception as e:
            raise SwitchErrorStates(e).switch()



import unittest
from thepeer import main  # type: ignore

thepeer_test_suites = main.ThepeerInit("pssk_test_vwww1yvvpymamtut26x5tvpx1znrcmeis2k0kvcmwzjax")


class ThePeerInitMethods(unittest.TestCase):
    def test_validate_signature(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.validate_signature(
                "I love the peer", "27154e4751dc49d1b40281b18deecd4fd2392e43"
            ),
            True,
        )

    def test_index_user(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.index_user(
                "Osagie Iyayi", "iyayiemmanuel1@gmail.com", "iyayiemmanuel1@gmail.com"
            ),
            {"message": "identifier exists"},
        )

    def test_view_user(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.view_user("3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd"),
            {
                "indexed_user": {
                    "name": "Osagie Iyayi",
                    "identifier": "iyayiemmanuel1@gmail.com",
                    "identifier_type": "email",
                    "email": "iyayiemmanuel1@gmail.com",
                    "reference": "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd",
                    "created_at": "2022-04-30T21:30:29.000000Z",
                    "updated_at": "2022-05-04T07:37:45.000000Z",
                }
            },
        )

    def test_all_users(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.all_users(),
            {
                "indexed_users": {
                    "data": [
                        {
                            "name": "Osagie Iyayi",
                            "identifier": "iyayiemmanuel1@gmail.com",
                            "identifier_type": "email",
                            "email": "iyayiemmanuel1@gmail.com",
                            "reference": "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd",
                            "created_at": "2022-04-30T21:30:29.000000Z",
                            "updated_at": "2022-05-04T07:37:45.000000Z",
                        },
                        {
                            "name": "Osagie Iyayi",
                            "identifier": "iyayiemmanuel1@gmail.com",
                            "identifier_type": "email",
                            "email": "iyayiemmanuel1@gmail.com",
                            "reference": "d2cb0c2c-7bd4-40a0-9744-824fbce176b7",
                            "created_at": "2022-05-04T07:37:44.000000Z",
                            "updated_at": "2022-05-04T07:37:44.000000Z",
                        },
                    ],
                    "meta": {"page": 1, "total": 2, "pageCount": 1, "perPage": 15},
                }
            },
        )

    def test_user_links(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.get_user_links("3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd"),
            {"links": []},
        )

    def test_single_link(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.get_single_link("3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd"),
            {"message": "link not found"},
        )

    def test_charge_link(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.charge_link(
                "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd", 2000, "try thepeer"
            ),
            {
                "message": "The given data was invalid.",
                "errors": {"amount": ["The amount must be at least 10000."]},
            },
        )

    def test_charge_link_2(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.charge_link(
                "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd", 10000, "try thepeer"
            ),
            {"message": "link not found"},
        )

    def test_authorize_direct_charge(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.authorize_direct_charge(
                "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd", "charge"
            ),
            {
                "message": "The given data was invalid.",
                "errors": {"event": ["The selected event is invalid."]},
            },
        )

    def test_transaction_detail(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.get_transaction_detail("3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd"),
            {"message": "transaction not found"},
        )

    def test_refund_transaction(self):
        self.assertEqual.__self__.maxDiff = None
        self.assertEqual(
            thepeer_test_suites.refund_transaction(
                "3bbb0fbf-82fa-48a0-80eb-d2c0338fe7dd", "possibly fraudulent"
            ),
            {"message": "this feature is only available on live mode"},
        )



from .base import BaseException as error


class ForbiddenException(error):
    """_summary_: ForbiddenException
    _description_: The forbidden exception for thepeer sdk
    _usage_:
        raise ForbiddenException(errors)
    _example_:
        raise ForbiddenException({message: "you shall not pass"})
    """

    def __init__(self, errors):
        super().__init__(errors)



from .base import BaseException as error


class ServerErrorException(error):
    """_summary_: ServerErrorException
    _description_: The server error exception for thepeer sdk
    _usage_:
        raise ServerErrorException(errors)
    _example_:
        raise ServerErrorException({message: "We played a little bit too much and broke something"})
    """

    def __init__(self, errors):
        super().__init__(errors)



from .types.forbidden import ForbiddenException  # type: ignore
from .types.notacceptable import NotAcceptableException  # type: ignore
from .types.notfound import NotFoundException  # type: ignore
from .types.servererror import ServerErrorException  # type: ignore
from .types.serviceunavailable import ServiceUnavailableException  # type: ignore
from .types.unauthorized import UnauthorizedException  # type: ignore
from .types.unprocessableentity import UnprocessableEntityException  # type: ignore


class SwitchErrorStates:
    """_summary_
    A class for switching between various error status codes returned from the peer servers
    """

    def __init__(self, error):
        self.errordata = error.response.data.error

    def switch(self):
        status = self.error.response.status
        error_message = self.error.response.data.message
        if status == 401:
            raise UnauthorizedException({"message": error_message})
        elif status == 403:
            raise ForbiddenException({"message": error_message})
        elif status == 404:
            raise NotFoundException({"message": error_message})
        elif status == 406:
            raise NotAcceptableException({"message": error_message})
        elif status == 422:
            raise UnprocessableEntityException({"message": error_message})
        elif status == 500:
            raise ServerErrorException({"message": error_message})
        elif status == 503:
            raise ServiceUnavailableException({"message": error_message})



import pathlib
import setuptools  # noqa: F401
from distutils.core import setup


# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


setup(
    name="pythepeer",
    version="0.0.1",
    description="official python sdk for interacting with thepeer payment processing \
    infrastructure",
    author="Osagie Iyayi",
    packages=["thepeer"],
    author_email="iyayiemmanuel1@gmail.com",
    url="https://github.com/thepeerstack/python-sdk",
    license="MIT",
    install_requires=["httpx"],
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        # as the current state of your package
        "Intended Audience :: Developers",  # Define that your audience are developers
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",  # Again, pick a license
        "Programming Language :: Python :: 3",
        # Specify which python versions that you want to support
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries",
    ],
    long_description=README,
    long_description_content_type="text/markdown",
    keywords=["python", "fintech", "peer-to-peer"],
    zip_safe=False,
)



from .base import BaseException as error


class NotAcceptableException(error):
    """_summary_: NotAcceptableException
    _description_: The not acceptable exception for thepeer sdk
    _usage_:
        raise NotAcceptableException(errors)
    _example_:
        raise NotAcceptableException({message: "you need to work to get some money"})
    """

    def __init__(self, errors):
        super().__init__(errors)



from .base import BaseException as error


class UnprocessableEntityException(error):
    """_summary_: UnprocessableEntityException
    _description_: The unprocessable entity exception for thepeer sdk
    _usage_:
        raise UnprocessableEntityException(errors)
    _example_:
        raise UnprocessableEntityException({message: "The given data was invalid.",
    "errors": {
        "name": [
            "The name field is required."
        ],
        "email": [
            "The email field is required."
        ],
        "identifier": [
            "The identifier field is required."
        ]
    }})
    """

    def __init__(self, errors):
        super().__init__(errors)
